<?php

namespace Drupal\simple_survey\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Drupal\simple_survey\Service\BigQueryConnect;
use Drupal\simple_survey\Service\ResponseExportService;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\simple_survey\Entity\SimpleSurvey;
use Drupal\simple_survey\Entity\SimpleSurveyInterface;

/**
 * Class SimpleSurveyController.
 *
 * @package Drupal\simple_survey\Controller
 */
class SimpleSurveyController extends ControllerBase {

  /**
   * The modules path.
   *
   * @var string
   */
  private $modulePath = '';

  /**
   * The modules machine name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal\Core\Messenger\MessengerInterface definition.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * Drupal\Core\Logger\LoggerChannelFactory definition.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\Service\BigQueryConnect
   */
  protected $bigQuery;

  /**
   * Response export service.
   *
   * @var \Drupal\simple_survey\ResponseExportService
   */
  protected $responseExport;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Simple Survery Controller constructor.
   *
   * @todo Look into creating a service to handle DI. Doing a janky call in feedbackjs.module
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The Drupal messenger service.
   * @param Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Drupal date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\simple_survey\Service\BigQueryConnect $big_query
   *   The BigQuery Connection service.
   * @param \Drupal\simple_survey\ResponseExportService $response_export_service
   *   The response export service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The Drupal time service.
   */
  public function __construct(
    MessengerInterface $messenger,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $logger_factory,
    BigQueryConnect $big_query,
    ResponseExportService $response_export_service,
    TimeInterface $time) {
    $this->messenger = $messenger;
    $this->dateFormatter = $dateFormatter;
    $this->logger = $logger_factory->get($this->moduleName);
    $this->bigQuery = $big_query;
    $this->responseExport = $response_export_service;
    $this->time = $time;

    $this->modulePath = drupal_get_path('module', $this->moduleName);
  }

  /**
   * Dependency Injection.
   *
   * @inheritDoc
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('logger.factory'),
      $container->get('simple_survey.big_query'),
      $container->get('simple_survey.response_export'),
      $container->get('datetime.time')
    );
  }

  /**
   * Displays the response as CSV.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Returns a JSON Response.
   */
  public function responseOutput(SimpleSurveyInterface $simple_survey) {
    $fileTime = $this->dateFormatter->format($this->time->getRequestTime(), 'custom', 'YdmHis');

    $headerResponse = new Response();
    $headerResponse->headers->set('Content-Type', 'test/csv; charset=utf-8');
    $headerResponse->headers->set('Content-Disposition', 'attachment; filename="'
     . $simple_survey->id() . '-survey-export-' . $fileTime . '.csv"');
    $headerResponse->sendHeaders();

    $this->responseExport->survey = $simple_survey;
    $this->responseExport->convertDataToCsv(FALSE);

    return $headerResponse;
  }

  /**
   * Submits all data to a bigQuery table via cron.
   */
  public function bigQueryCronUpdate() {
    if (!$this->bigQuery->checkConfigured()) {
      $this->logger->alert('BigQuery is not configured.');
    }
    else {
      $surveys = SimpleSurvey::loadMultiple();

      foreach ($surveys as $survey) {
        if (!$survey->isSurveyActive()) {
          continue;
        }

        $this->responseExport->survey = $survey;
        $csv = $this->responseExport->convertDataToCsv();

        $updated = $this->bigQuery->updateTable('survey_tool', 'survey_response', $csv);

        // Check if the job has errors.
        if ($updated) {
          $this->logger->info('Completed Updating BigQuery Table for ' . $survey->label());
        }
      }
    }
  }

  /**
   * Manual submission to BigQuery.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Returns response object.
   */
  public function bigQueryManualUpdate(SimpleSurveyInterface $simple_survey) {
    $this->bigQuery->setSurvey($simple_survey);
    $this->responseExport->survey = $simple_survey;

    if (!$this->bigQuery->checkConfigured()) {
      $this->messenger->addWarning('BigQuery is not configured.');

      return $this->redirect('entity.simple_survey.edit_form', [
        'simple_survey' => $simple_survey->id(),
      ]);
    }

    $csv = $this->responseExport->convertDataToCsv();

    $updated = $this->bigQuery->updateTable(
      $simple_survey->get('bigQueryDataset'), $simple_survey->get('bigQueryTable'), $csv);

    // Check if the job has errors.
    if (!$updated) {
      $this->messenger->addMessage($this->t('Data could not be sent.'));

      return $this->redirect('entity.simple_survey.edit_form', [
        'simple_survey' => $simple_survey->id(),
      ]);
    }
    else {
      $this->messenger->addMessage($this->t('Data sent to BigQuery.'));

      return $this->redirect('entity.simple_survey.edit_form', [
        'simple_survey' => $simple_survey->id(),
      ]);
    }
  }

}
