<?php

namespace Drupal\simple_survey\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use SplFileObject;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Survey Response convert and export service.
 */
class ResponseExportService implements ContainerInjectionInterface {

  /**
   * The module name.
   *
   * @var string
   */
  private $moduleName = 'simple_survey';

  /**
   * Drupal config service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * Drupal logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Drupal messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The Drupal Time Service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The Entity Type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current survey.
   *
   * @var \Drupal\simple_survey\Entity\SimpleSurvey
   */
  public $survey;

  /**
   * PHP File Object.
   *
   * @var \SplFileObject
   */
  public $fileObject;

  /**
   * Service config and DI.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   Drupal Messenger service.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   Date formatter service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   Drupal logger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Drupal config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type Manager service.
   */
  public function __construct(
    MessengerInterface $messenger,
    DateFormatter $dateFormatter,
    LoggerChannelFactoryInterface $loggerFactory,
    ConfigFactoryInterface $configFactory,
    EntityTypeManagerInterface $entity_type_manager,
    TimeInterface $time
  ) {
    $this->messenger = $messenger;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $loggerFactory->get($this->moduleName);
    $this->time = $time;

    $this->simpleSurveyConfig = $this->configFactory->getEditable($this->moduleName . '.settings');
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('datetime.time')
    );
  }

  /**
   * Builds the csv header row and sets column keys.
   *
   * @return array
   *   Returns an array of header data.
   */
  private function outputHeaderRow() {

    $headerColumnKeys = [
      'drupal_id',
      'cookie_uuid',
      'site_path',
      'langcode',
    ];

    $headerColumns = [
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Drupal ID'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'User'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Page Submitted On'))),
      preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Language'))),
    ];

    $query = $this->entityTypeManager->getStorage('survey_question')->getQuery();

    if ($this->survey) {
      $query->condition('survey', $this->survey->id());
    }

    $nids = $query
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyQuestion $questions */
    $questions = $this->entityTypeManager->getStorage('survey_question')
      ->loadMultiple($nids);

    foreach ($questions as $question) {
      if ($question->get('question_type')->value == 'normal') {
        $sanitizeQuestion = strip_tags($question->get('question')->value);
        $sanitizeQuestion = str_replace('&nbsp;', ' ', $sanitizeQuestion);
        $sanitizeQuestion = str_replace(["\r", "\n"], '', $sanitizeQuestion);
        $headerColumns[] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', $sanitizeQuestion)));
        $headerColumnKeys[] = $question->id();
      }
    }

    $headerColumns[] = preg_replace('/[^A-Za-z0-9_]/', '', strtolower(str_replace(' ', '_', 'Submission was created')));
    $headerColumnKeys[] = 'created';
    $this->fileObject->fputcsv($headerColumns);

    return $headerColumnKeys;
  }

  /**
   * Builds a response data rray.
   */
  private function outputResponseData() {
    $headerColumnKeys = $this->outputHeaderRow();

    $query = $this->entityTypeManager->getStorage('survey_response')->getQuery();

    if ($this->survey) {
      $query->condition('survey', $this->survey->id());
    }

    $srids = $query
      ->sort('created', 'DESC')
      ->execute();

    /** @var Drupal/simple_survey/Entity/SurveyResponse $survey_responses */
    $survey_responses = $this->entityTypeManager->getStorage('survey_response')
      ->loadMultiple($srids);

    foreach ($survey_responses as $survey_response) {
      $outputResponse = [];

      foreach ($headerColumnKeys as $key) {
        $outputResponse[$key] = NULL;
      }

      $outputResponse['drupal_id'] = $survey_response->id();
      $outputResponse['cookie_uuid'] = $survey_response->get('cookie_uuid')->value;
      $outputResponse['site_path'] = $survey_response->get('site_path')->value;
      $outputResponse['langcode'] = $survey_response->get('langcode')->value;
      $outputResponse['created'] = $this->dateFormatter->format($survey_response->get('created')->value, 'custom', 'Y-m-d H:i:s');

      $query = $this->entityTypeManager->getStorage('survey_answer')->getQuery();

      $saids = $query
        ->condition('parent_uuid', $survey_response->get('uuid')->value)
        ->execute();

      /** @var Drupal/simple_survey/Entity/SurveyAnswer $survey_answers */
      $survey_answers = $this->entityTypeManager->getStorage('survey_answer')
        ->loadMultiple($saids);

      foreach ($survey_answers as $survey_answer) {
        $outputResponse[$survey_answer->get('question_id')->value] = $survey_answer->get('answer')->value;
      }

      $this->fileObject->fputcsv($outputResponse);
    }

  }

  /**
   * Converts survey responses to CSV.
   *
   * @param bool $collectOutput
   *   Flag to collect output.
   *
   * @return string
   *   Returns converted data.
   */
  public function convertDataToCsv($collectOutput = TRUE) {
    $fileTime = $this->dateFormatter->format($this->time->getRequestTime(), 'custom', 'YdmHis');
    $file = \Drupal::service('file_system')->realpath("public://")
      . '/' . $this->survey->id() . '-survey-export-' . $fileTime . '.csv';

    $stream = $collectOutput ? $file : 'php://output';
    $mode = $collectOutput ? 'w+' : 'r+';

    $this->fileObject = new SplFileObject($stream, $mode);

    $this->outputResponseData();

    if ($collectOutput) {
      $csv = file_get_contents($file);
      unlink($file);
      return $csv;
    }

  }

}
